# encoding: utf-8

"""lxml custom element classes for picture-related XML elements."""

from __future__ import absolute_import, division, print_function, unicode_literals

from .. import parse_xml
from ..ns import nsdecls
from .shared import BaseShapeElement
from ..xmlchemy import BaseOxmlElement, OneAndOnlyOne


class CT_Picture(BaseShapeElement):
    """
    ``<p:pic>`` element, which represents a picture shape (an image placement
    on a slide).
    """

    nvPicPr = OneAndOnlyOne("p:nvPicPr")
    blipFill = OneAndOnlyOne("p:blipFill")
    spPr = OneAndOnlyOne("p:spPr")

    @property
    def blip_rId(self):
        """Value of `p:blipFill/a:blip/@r:embed`.

        Returns |None| if not present.
        """
        blip = self.blipFill.blip
        if blip is not None and blip.rEmbed is not None:
            return blip.rEmbed
        return None

    def crop_to_fit(self, image_size, view_size):
        """
        Set cropping values in `p:blipFill/a:srcRect` such that an image of
        *image_size* will stretch to exactly fit *view_size* when its aspect
        ratio is preserved.
        """
        self.blipFill.crop(self._fill_cropping(image_size, view_size))

    def get_or_add_ln(self):
        """
        Return the <a:ln> grandchild element, newly added if not present.
        """
        return self.spPr.get_or_add_ln()

    @property
    def ln(self):
        """
        ``<a:ln>`` grand-child element or |None| if not present
        """
        return self.spPr.ln

    @classmethod
    def new_ph_pic(cls, id_, name, desc, rId):
        """
        Return a new `p:pic` placeholder element populated with the supplied
        parameters.
        """
        return parse_xml(cls._pic_ph_tmpl() % (id_, name, desc, rId))

    @classmethod
    def new_pic(cls, id_, name, desc, rId, left, top, width, height):
        """
        Return a new ``<p:pic>`` element tree configured with the supplied
        parameters.
        """
        xml = cls._pic_tmpl() % (id_, name, desc, rId, left, top, width, height)
        pic = parse_xml(xml)
        return pic

    @classmethod
    def new_video_pic(
        cls, shape_id, shape_name, video_rId, media_rId, poster_frame_rId, x, y, cx, cy
    ):
        """Return a new `p:pic` populated with the specified video."""
        return parse_xml(
            cls._pic_video_tmpl()
            % (
                shape_id,
                shape_name,
                video_rId,
                media_rId,
                poster_frame_rId,
                x,
                y,
                cx,
                cy,
            )
        )

    @property
    def srcRect_b(self):
        """Value of `p:blipFill/a:srcRect/@b` or 0.0 if not present."""
        return self._srcRect_x("b")

    @srcRect_b.setter
    def srcRect_b(self, value):
        self.blipFill.get_or_add_srcRect().b = value

    @property
    def srcRect_l(self):
        """Value of `p:blipFill/a:srcRect/@l` or 0.0 if not present."""
        return self._srcRect_x("l")

    @srcRect_l.setter
    def srcRect_l(self, value):
        self.blipFill.get_or_add_srcRect().l = value  # noqa

    @property
    def srcRect_r(self):
        """Value of `p:blipFill/a:srcRect/@r` or 0.0 if not present."""
        return self._srcRect_x("r")

    @srcRect_r.setter
    def srcRect_r(self, value):
        self.blipFill.get_or_add_srcRect().r = value

    @property
    def srcRect_t(self):
        """Value of `p:blipFill/a:srcRect/@t` or 0.0 if not present."""
        return self._srcRect_x("t")

    @srcRect_t.setter
    def srcRect_t(self, value):
        self.blipFill.get_or_add_srcRect().t = value

    def _fill_cropping(self, image_size, view_size):
        """
        Return a (left, top, right, bottom) 4-tuple containing the cropping
        values required to display an image of *image_size* in *view_size*
        when stretched proportionately. Each value is a percentage expressed
        as a fraction of 1.0, e.g. 0.425 represents 42.5%. *image_size* and
        *view_size* are each (width, height) pairs.
        """

        def aspect_ratio(width, height):
            return width / height

        ar_view = aspect_ratio(*view_size)
        ar_image = aspect_ratio(*image_size)

        if ar_view < ar_image:  # image too wide
            crop = (1.0 - (ar_view / ar_image)) / 2.0
            return (crop, 0.0, crop, 0.0)
        if ar_view > ar_image:  # image too tall
            crop = (1.0 - (ar_image / ar_view)) / 2.0
            return (0.0, crop, 0.0, crop)
        return (0.0, 0.0, 0.0, 0.0)

    @classmethod
    def _pic_ph_tmpl(cls):
        return (
            "<p:pic %s>\n"
            "  <p:nvPicPr>\n"
            '    <p:cNvPr id="%%d" name="%%s" descr="%%s"/>\n'
            "    <p:cNvPicPr>\n"
            '      <a:picLocks noGrp="1" noChangeAspect="1"/>\n'
            "    </p:cNvPicPr>\n"
            "    <p:nvPr/>\n"
            "  </p:nvPicPr>\n"
            "  <p:blipFill>\n"
            '    <a:blip r:embed="%%s"/>\n'
            "    <a:stretch>\n"
            "      <a:fillRect/>\n"
            "    </a:stretch>\n"
            "  </p:blipFill>\n"
            "  <p:spPr/>\n"
            "</p:pic>" % nsdecls("p", "a", "r")
        )

    @classmethod
    def _pic_tmpl(cls):
        return (
            "<p:pic %s>\n"
            "  <p:nvPicPr>\n"
            '    <p:cNvPr id="%%d" name="%%s" descr="%%s"/>\n'
            "    <p:cNvPicPr>\n"
            '      <a:picLocks noChangeAspect="1"/>\n'
            "    </p:cNvPicPr>\n"
            "    <p:nvPr/>\n"
            "  </p:nvPicPr>\n"
            "  <p:blipFill>\n"
            '    <a:blip r:embed="%%s"/>\n'
            "    <a:stretch>\n"
            "      <a:fillRect/>\n"
            "    </a:stretch>\n"
            "  </p:blipFill>\n"
            "  <p:spPr>\n"
            "    <a:xfrm>\n"
            '      <a:off x="%%d" y="%%d"/>\n'
            '      <a:ext cx="%%d" cy="%%d"/>\n'
            "    </a:xfrm>\n"
            '    <a:prstGeom prst="rect">\n'
            "      <a:avLst/>\n"
            "    </a:prstGeom>\n"
            "  </p:spPr>\n"
            "</p:pic>" % nsdecls("a", "p", "r")
        )

    @classmethod
    def _pic_video_tmpl(cls):
        return (
            "<p:pic %s>\n"
            "  <p:nvPicPr>\n"
            '    <p:cNvPr id="%%d" name="%%s">\n'
            '      <a:hlinkClick r:id="" action="ppaction://media"/>\n'
            "    </p:cNvPr>\n"
            "    <p:cNvPicPr>\n"
            '      <a:picLocks noChangeAspect="1"/>\n'
            "    </p:cNvPicPr>\n"
            "    <p:nvPr>\n"
            '      <a:videoFile r:link="%%s"/>\n'
            "      <p:extLst>\n"
            '        <p:ext uri="{DAA4B4D4-6D71-4841-9C94-3DE7FCFB9230}">\n'
            '          <p14:media xmlns:p14="http://schemas.microsoft.com/of'
            'fice/powerpoint/2010/main" r:embed="%%s"/>\n'
            "        </p:ext>\n"
            "      </p:extLst>\n"
            "    </p:nvPr>\n"
            "  </p:nvPicPr>\n"
            "  <p:blipFill>\n"
            '    <a:blip r:embed="%%s"/>\n'
            "    <a:stretch>\n"
            "      <a:fillRect/>\n"
            "    </a:stretch>\n"
            "  </p:blipFill>\n"
            "  <p:spPr>\n"
            "    <a:xfrm>\n"
            '      <a:off x="%%d" y="%%d"/>\n'
            '      <a:ext cx="%%d" cy="%%d"/>\n'
            "    </a:xfrm>\n"
            '    <a:prstGeom prst="rect">\n'
            "      <a:avLst/>\n"
            "    </a:prstGeom>\n"
            "  </p:spPr>\n"
            "</p:pic>" % nsdecls("a", "p", "r")
        )

    def _srcRect_x(self, attr_name):
        """
        Value of `p:blipFill/a:srcRect/@{attr_name}` or 0.0 if not present.
        """
        srcRect = self.blipFill.srcRect
        if srcRect is None:
            return 0.0
        return getattr(srcRect, attr_name)


class CT_PictureNonVisual(BaseOxmlElement):
    """
    ``<p:nvPicPr>`` element, containing non-visual properties for a picture
    shape.
    """

    cNvPr = OneAndOnlyOne("p:cNvPr")
    nvPr = OneAndOnlyOne("p:nvPr")



# encoding: utf-8

"""Gherkin step implementations for FillFormat-related features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, then, when

from pptx import Presentation
from pptx.enum.dml import MSO_FILL, MSO_PATTERN  # noqa

from helpers import test_pptx


# given ====================================================


@given("a FillFormat object as fill")
def given_a_FillFormat_object_as_fill(context):
    fill = Presentation(test_pptx("dml-fill")).slides[0].shapes[0].fill
    context.fill = fill


@given("a FillFormat object as fill having {pattern} fill")
def given_a_FillFormat_object_as_fill_having_pattern(context, pattern):
    shape_idx = {"no pattern": 0, "MSO_PATTERN.DIVOT": 1, "MSO_PATTERN.WAVE": 2}[
        pattern
    ]
    slide = Presentation(test_pptx("dml-fill")).slides[1]
    fill = slide.shapes[shape_idx].fill
    context.fill = fill


@given("{type} FillFormat object as fill")
def given_type_FillFormat_object_as_fill(context, type):
    shape_idx = {
        "an inheriting": 0,
        "a no-fill": 1,
        "a solid": 2,
        "a picture": 3,
        "a gradient": 4,
        "a patterned": 5,
    }[type]
    shape = Presentation(test_pptx("dml-fill")).slides[0].shapes[shape_idx]
    context.fill = shape.fill


@given("a _GradientStop object as stop")
def given_a_GradientStop_object_as_stop(context):
    shape = Presentation(test_pptx("dml-fill")).slides[0].shapes[4]
    context.stop = shape.fill.gradient_stops[0]


# when =====================================================


@when("I assign {value} to fill.gradient_angle")
def when_I_assign_value_to_fill_gradient_angle(context, value):
    context.fill.gradient_angle = eval(value)


@when("I assign {value} to fill.pattern")
def when_I_assign_value_to_fill_pattern(context, value):
    pattern = {
        "None": None,
        "MSO_PATTERN.CROSS": MSO_PATTERN.CROSS,
        "MSO_PATTERN.DIVOT": MSO_PATTERN.DIVOT,
        "MSO_PATTERN.WAVE": MSO_PATTERN.WAVE,
    }[value]
    context.fill.pattern = pattern


@when("I assign {value} to stop.position")
def when_I_assign_value_to_stop_position(context, value):
    context.stop.position = eval(value)


@when("I call fill.background()")
def when_I_call_fill_background(context):
    context.fill.background()


@when("I call fill.gradient()")
def when_I_call_fill_gradient(context):
    context.fill.gradient()


@when("I call fill.patterned()")
def when_I_call_fill_patterned(context):
    context.fill.patterned()


@when("I call fill.solid()")
def when_I_call_fill_solid(context):
    context.fill.solid()


# then =====================================================


@then("fill.back_color is a ColorFormat object")
def then_fill_back_color_is_a_ColorFormat_object(context):
    actual_value = context.fill.back_color.__class__.__name__
    expected_value = "ColorFormat"
    assert actual_value == expected_value, (
        "fill.back_color is a %s object" % actual_value
    )


@then("fill.fore_color is a ColorFormat object")
def then_fill_fore_color_is_a_ColorFormat_object(context):
    actual_value = context.fill.fore_color.__class__.__name__
    expected_value = "ColorFormat"
    assert actual_value == expected_value, (
        "fill.fore_color is a %s object" % actual_value
    )


@then("fill.gradient_angle == {value}")
def then_fill_gradient_angle_eq_value(context, value):
    expected_value = round(eval(value), 2)
    actual_value = round(context.fill.gradient_angle, 2)
    assert actual_value == expected_value, "fill.gradient_angle == %s" % actual_value


@then("fill.gradient_stops is a _GradientStops object")
def then_fill_gradient_stops_is_a_GradientStops_object(context):
    expected_value = "_GradientStops"
    actual_value = context.fill.gradient_stops.__class__.__name__
    assert actual_value == expected_value, (
        "fill.gradient_stops is a %s object" % actual_value
    )


@then("fill.pattern is {value}")
def then_fill_pattern_is_value(context, value):
    fill_pattern = context.fill.pattern
    expected_value = {
        "None": None,
        "MSO_PATTERN.CROSS": MSO_PATTERN.CROSS,
        "MSO_PATTERN.DIVOT": MSO_PATTERN.DIVOT,
        "MSO_PATTERN.WAVE": MSO_PATTERN.WAVE,
    }[value]
    assert fill_pattern == expected_value, "expected fill pattern %s, got %s" % (
        expected_value,
        fill_pattern,
    )


@then("fill.type == {value}")
def then_fill_type_eq_value(context, value):
    expected_value = eval(value)
    actual_value = context.fill.type
    assert actual_value == expected_value, "fill.type is %s" % actual_value


@then("stop.color is a ColorFormat object")
def then_stop_color_is_a_ColorFormat_object(context):
    expected_value = "ColorFormat"
    actual_value = context.stop.color.__class__.__name__
    assert actual_value == expected_value, "stop.color is a %s object" % actual_value


@then("stop.position == {value}")
def then_stop_position_eq_value(context, value):
    expected_value = eval(value)
    actual_value = context.stop.position
    assert actual_value == expected_value, "stop.position == %s" % actual_value



# encoding: utf-8

"""Unit test suite for pptx.shapes.connector module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.dml.line import LineFormat
from pptx.enum.shapes import MSO_SHAPE_TYPE
from pptx.shapes.base import BaseShape
from pptx.shapes.connector import Connector
from pptx.util import Emu

from ..unitutil.cxml import element, xml
from ..unitutil.mock import instance_mock, method_mock


class DescribeConnector(object):
    """Unit-test suite for `pptx.shapes.connector.Connector`."""

    def it_knows_its_begin_point_x_location(self, begin_x_get_fixture):
        connector, expected_value = begin_x_get_fixture
        begin_x = connector.begin_x
        assert isinstance(begin_x, Emu)
        assert connector.begin_x == expected_value

    def it_can_change_its_begin_point_x_location(self, begin_x_set_fixture):
        connector, new_x, expected_xml = begin_x_set_fixture
        connector.begin_x = new_x
        assert connector._element.xml == expected_xml

    def it_knows_its_begin_point_y_location(self, begin_y_get_fixture):
        connector, expected_value = begin_y_get_fixture
        begin_y = connector.begin_y
        assert isinstance(begin_y, Emu)
        assert connector.begin_y == expected_value

    def it_can_change_its_begin_point_y_location(self, begin_y_set_fixture):
        connector, new_y, expected_xml = begin_y_set_fixture
        connector.begin_y = new_y
        assert connector._element.xml == expected_xml

    def it_knows_its_end_point_x_location(self, end_x_get_fixture):
        connector, expected_value = end_x_get_fixture
        end_x = connector.end_x
        assert isinstance(end_x, Emu)
        assert connector.end_x == expected_value

    def it_can_change_its_end_point_x_location(self, end_x_set_fixture):
        connector, new_x, expected_xml = end_x_set_fixture
        connector.end_x = new_x
        assert connector._element.xml == expected_xml

    def it_knows_its_end_point_y_location(self, end_y_get_fixture):
        connector, expected_value = end_y_get_fixture
        end_y = connector.end_y
        assert isinstance(end_y, Emu)
        assert connector.end_y == expected_value

    def it_can_change_its_end_point_y_location(self, end_y_set_fixture):
        connector, new_y, expected_xml = end_y_set_fixture
        connector.end_y = new_y
        assert connector._element.xml == expected_xml

    def it_can_connect_its_begin_point_to_a_shape(self, begin_conn_fixture):
        connector, shape, cxn_idx = begin_conn_fixture

        connector.begin_connect(shape, cxn_idx)

        connector._connect_begin_to.assert_called_once_with(connector, shape, cxn_idx)
        connector._move_begin_to_cxn.assert_called_once_with(connector, shape, cxn_idx)

    def it_can_connect_its_end_point_to_a_shape(self, end_conn_fixture):
        connector, shape, cxn_idx = end_conn_fixture

        connector.end_connect(shape, cxn_idx)

        connector._connect_end_to.assert_called_once_with(connector, shape, cxn_idx)
        connector._move_end_to_cxn.assert_called_once_with(connector, shape, cxn_idx)

    def it_provides_access_to_its_line_format(self):
        connector = Connector(element("p:cxnSp/p:spPr"), None)

        line = connector.line

        assert isinstance(line, LineFormat)
        # exercise line to test parent interface, .ln and .get_or_add_ln()
        line.width = 91440
        assert line.width == 91440

    def it_knows_its_shape_type(self):
        assert Connector(None, None).shape_type == MSO_SHAPE_TYPE.LINE

    def it_connects_its_begin_point_to_help(self, connect_begin_fixture):
        connector, shape, cxn_idx, expected_xml = connect_begin_fixture
        connector._connect_begin_to(shape, cxn_idx)
        assert connector._element.xml == expected_xml

    def it_connects_its_end_point_to_help(self, connect_end_fixture):
        connector, shape, cxn_idx, expected_xml = connect_end_fixture
        connector._connect_end_to(shape, cxn_idx)
        assert connector._element.xml == expected_xml

    def it_moves_its_begin_point_to_help(self, move_begin_fixture):
        connector, shape, cxn_idx, expected_xml = move_begin_fixture
        connector._move_begin_to_cxn(shape, cxn_idx)
        assert connector._element.xml == expected_xml

    def it_moves_its_end_point_to_help(self, move_end_fixture):
        connector, shape, cxn_idx, expected_xml = move_end_fixture
        connector._move_end_to_cxn(shape, cxn_idx)
        assert connector._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def begin_conn_fixture(self, _connect_begin_to_, _move_begin_to_cxn_):
        connector = Connector(None, None)
        shape, cxn_idx = 42, 24
        return connector, shape, cxn_idx

    @pytest.fixture(params=[(42, 24, False, 42), (24, 42, True, 66)])
    def begin_x_get_fixture(self, request):
        x, cx, flipH, expected_value = request.param
        cxnSp = element(
            "p:cxnSp/p:spPr/a:xfrm{flipH=%d}/(a:off{x=%d,y=6},a:ext{cx=%d,cy"
            "=32})" % (flipH, x, cx)
        )
        connector = Connector(cxnSp, None)
        return connector, expected_value

    @pytest.fixture(
        params=[
            (
                "a:xfrm/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                5,
                "a:xfrm/(a:off{x=5,y=1},a:ext{cx=15,cy=1})",
            ),
            (
                "a:xfrm/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                15,
                "a:xfrm/(a:off{x=15,y=1},a:ext{cx=5,cy=1})",
            ),
            (
                "a:xfrm/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                25,
                "a:xfrm{flipH=1}/(a:off{x=20,y=1},a:ext{cx=5,cy=1})",
            ),
            (
                "a:xfrm{flipH=1}/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                25,
                "a:xfrm{flipH=1}/(a:off{x=10,y=1},a:ext{cx=15,cy=1})",
            ),
            (
                "a:xfrm{flipH=1}/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                15,
                "a:xfrm{flipH=1}/(a:off{x=10,y=1},a:ext{cx=5,cy=1})",
            ),
            (
                "a:xfrm{flipH=1}/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                5,
                "a:xfrm/(a:off{x=5,y=1},a:ext{cx=5,cy=1})",
            ),
        ]
    )
    def begin_x_set_fixture(self, request):
        xfrm_cxml, new_x, expected_cxml = request.param
        tmpl = "p:cxnSp/p:spPr/%s"
        cxnSp = element(tmpl % xfrm_cxml)
        expected_xml = xml(tmpl % expected_cxml)
        connector = Connector(cxnSp, None)
        return connector, new_x, expected_xml

    @pytest.fixture(params=[(40, 60, False, 40), (50, 42, True, 92)])
    def begin_y_get_fixture(self, request):
        y, cy, flipV, expected_value = request.param
        cxnSp = element(
            "p:cxnSp/p:spPr/a:xfrm{flipV=%d}/(a:off{x=6,y=%d},a:ext{cx=32,cy"
            "=%d})" % (flipV, y, cy)
        )
        connector = Connector(cxnSp, None)
        return connector, expected_value

    @pytest.fixture(
        params=[
            (
                "a:xfrm/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                5,
                "a:xfrm/(a:off{x=1,y=5},a:ext{cx=1,cy=15})",
            ),
            (
                "a:xfrm/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                15,
                "a:xfrm/(a:off{x=1,y=15},a:ext{cx=1,cy=5})",
            ),
            (
                "a:xfrm/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                25,
                "a:xfrm{flipV=1}/(a:off{x=1,y=20},a:ext{cx=1,cy=5})",
            ),
            (
                "a:xfrm{flipV=1}/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                30,
                "a:xfrm{flipV=1}/(a:off{x=1,y=10},a:ext{cx=1,cy=20})",
            ),
            (
                "a:xfrm{flipV=1}/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                15,
                "a:xfrm{flipV=1}/(a:off{x=1,y=10},a:ext{cx=1,cy=5})",
            ),
            (
                "a:xfrm{flipV=1}/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                5,
                "a:xfrm/(a:off{x=1,y=5},a:ext{cx=1,cy=5})",
            ),
        ]
    )
    def begin_y_set_fixture(self, request):
        xfrm_cxml, new_y, expected_cxml = request.param
        tmpl = "p:cxnSp/p:spPr/%s"
        cxnSp = element(tmpl % xfrm_cxml)
        expected_xml = xml(tmpl % expected_cxml)
        connector = Connector(cxnSp, None)
        return connector, new_y, expected_xml

    @pytest.fixture(
        params=[
            (
                "p:cxnSp{a:b=c}/p:nvCxnSpPr/p:cNvCxnSpPr",
                "p:cxnSp{a:b=c}/p:nvCxnSpPr/p:cNvCxnSpPr/a:stCxn{id=42,idx=3}",
            ),
            (
                "p:cxnSp/p:nvCxnSpPr/p:cNvCxnSpPr/a:stCxn{id=66,idx=6}",
                "p:cxnSp/p:nvCxnSpPr/p:cNvCxnSpPr/a:stCxn{id=42,idx=3}",
            ),
        ]
    )
    def connect_begin_fixture(self, request, shape_):
        cxnSp_cxml, expected_cxml = request.param
        cxnSp = element(cxnSp_cxml)
        connector = Connector(cxnSp, None)
        shape_.shape_id, cxn_idx = 42, 3
        expected_xml = xml(expected_cxml)
        return connector, shape_, cxn_idx, expected_xml

    @pytest.fixture(
        params=[
            (
                "p:cxnSp{a:b=c}/p:nvCxnSpPr/p:cNvCxnSpPr",
                "p:cxnSp{a:b=c}/p:nvCxnSpPr/p:cNvCxnSpPr/a:endCxn{id=24,idx=2}",
            ),
            (
                "p:cxnSp/p:nvCxnSpPr/p:cNvCxnSpPr/a:endCxn{id=66,idx=6}",
                "p:cxnSp/p:nvCxnSpPr/p:cNvCxnSpPr/a:endCxn{id=24,idx=2}",
            ),
        ]
    )
    def connect_end_fixture(self, request, shape_):
        cxnSp_cxml, expected_cxml = request.param
        cxnSp = element(cxnSp_cxml)
        connector = Connector(cxnSp, None)
        shape_.shape_id, cxn_idx = 24, 2
        expected_xml = xml(expected_cxml)
        return connector, shape_, cxn_idx, expected_xml

    @pytest.fixture
    def end_conn_fixture(self, _connect_end_to_, _move_end_to_cxn_):
        connector = Connector(None, None)
        shape, cxn_idx = 42, 24
        return connector, shape, cxn_idx

    @pytest.fixture(params=[(21, 32, False, 53), (43, 54, True, 43)])
    def end_x_get_fixture(self, request):
        x, cx, flipH, expected_value = request.param
        cxnSp = element(
            "p:cxnSp/p:spPr/a:xfrm{flipH=%d}/(a:off{x=%d,y=6},a:ext{cx=%d,cy"
            "=60})" % (flipH, x, cx)
        )
        connector = Connector(cxnSp, None)
        return connector, expected_value

    @pytest.fixture(
        params=[
            (
                "a:xfrm/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                32,
                "a:xfrm/(a:off{x=10,y=1},a:ext{cx=22,cy=1})",
            ),
            (
                "a:xfrm/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                15,
                "a:xfrm/(a:off{x=10,y=1},a:ext{cx=5,cy=1})",
            ),
            (
                "a:xfrm/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                5,
                "a:xfrm{flipH=1}/(a:off{x=5,y=1},a:ext{cx=5,cy=1})",
            ),
            (
                "a:xfrm{flipH=1}/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                5,
                "a:xfrm{flipH=1}/(a:off{x=5,y=1},a:ext{cx=15,cy=1})",
            ),
            (
                "a:xfrm{flipH=1}/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                15,
                "a:xfrm{flipH=1}/(a:off{x=15,y=1},a:ext{cx=5,cy=1})",
            ),
            (
                "a:xfrm{flipH=1}/(a:off{x=10,y=1},a:ext{cx=10,cy=1})",
                28,
                "a:xfrm/(a:off{x=20,y=1},a:ext{cx=8,cy=1})",
            ),
        ]
    )
    def end_x_set_fixture(self, request):
        xfrm_cxml, new_x, expected_cxml = request.param
        tmpl = "p:cxnSp/p:spPr/%s"
        cxnSp = element(tmpl % xfrm_cxml)
        expected_xml = xml(tmpl % expected_cxml)
        connector = Connector(cxnSp, None)
        return connector, new_x, expected_xml

    @pytest.fixture(params=[(31, 42, False, 73), (53, 14, True, 53)])
    def end_y_get_fixture(self, request):
        y, cy, flipV, expected_value = request.param
        cxnSp = element(
            "p:cxnSp/p:spPr/a:xfrm{flipV=%d}/(a:off{x=6,y=%d},a:ext{cx=32,cy"
            "=%d})" % (flipV, y, cy)
        )
        connector = Connector(cxnSp, None)
        return connector, expected_value

    @pytest.fixture(
        params=[
            (
                "a:xfrm/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                28,
                "a:xfrm/(a:off{x=1,y=10},a:ext{cx=1,cy=18})",
            ),
            (
                "a:xfrm/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                13,
                "a:xfrm/(a:off{x=1,y=10},a:ext{cx=1,cy=3})",
            ),
            (
                "a:xfrm/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                4,
                "a:xfrm{flipV=1}/(a:off{x=1,y=4},a:ext{cx=1,cy=6})",
            ),
            (
                "a:xfrm{flipV=1}/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                6,
                "a:xfrm{flipV=1}/(a:off{x=1,y=6},a:ext{cx=1,cy=14})",
            ),
            (
                "a:xfrm{flipV=1}/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                12,
                "a:xfrm{flipV=1}/(a:off{x=1,y=12},a:ext{cx=1,cy=8})",
            ),
            (
                "a:xfrm{flipV=1}/(a:off{x=1,y=10},a:ext{cx=1,cy=10})",
                27,
                "a:xfrm/(a:off{x=1,y=20},a:ext{cx=1,cy=7})",
            ),
        ]
    )
    def end_y_set_fixture(self, request):
        xfrm_cxml, new_y, expected_cxml = request.param
        tmpl = "p:cxnSp/p:spPr/%s"
        cxnSp = element(tmpl % xfrm_cxml)
        expected_xml = xml(tmpl % expected_cxml)
        connector = Connector(cxnSp, None)
        return connector, new_y, expected_xml

    @pytest.fixture(
        params=[
            (0, "p:cxnSp/p:spPr/a:xfrm/(a:off{x=25,y=15},a:ext{cx=74,cy=123})"),
            (1, "p:cxnSp/p:spPr/a:xfrm/(a:off{x=10,y=33},a:ext{cx=89,cy=105})"),
            (2, "p:cxnSp/p:spPr/a:xfrm/(a:off{x=25,y=51},a:ext{cx=74,cy=87})"),
            (3, "p:cxnSp/p:spPr/a:xfrm/(a:off{x=40,y=33},a:ext{cx=59,cy=105})"),
        ]
    )
    def move_begin_fixture(self, request, shape_):
        cxn_idx, expected_cxml = request.param
        cxnSp = element("p:cxnSp/p:spPr/a:xfrm/(a:off{x=66,y=99},a:ext{cx=33,cy=39})")
        connector = Connector(cxnSp, None)
        shape_.left, shape_.top, shape_.width, shape_.height = 10, 15, 30, 36
        expected_xml = xml(expected_cxml)
        return connector, shape_, cxn_idx, expected_xml

    @pytest.fixture(
        params=[
            (0, "p:cxnSp/p:spPr/a:xfrm/(a:off{x=10,y=15},a:ext{cx=50,cy=10})"),
            (1, "p:cxnSp/p:spPr/a:xfrm/(a:off{x=10,y=15},a:ext{cx=40,cy=19})"),
            (2, "p:cxnSp/p:spPr/a:xfrm/(a:off{x=10,y=15},a:ext{cx=50,cy=28})"),
            (3, "p:cxnSp/p:spPr/a:xfrm/(a:off{x=10,y=15},a:ext{cx=60,cy=19})"),
        ]
    )
    def move_end_fixture(self, request, shape_):
        cxn_idx, expected_cxml = request.param
        cxnSp = element("p:cxnSp/p:spPr/a:xfrm/(a:off{x=10,y=15},a:ext{cx=10,cy=5})")
        connector = Connector(cxnSp, None)
        shape_.left, shape_.top, shape_.width, shape_.height = 50, 25, 20, 18
        expected_xml = xml(expected_cxml)
        return connector, shape_, cxn_idx, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _connect_begin_to_(self, request):
        return method_mock(request, Connector, "_connect_begin_to", autospec=True)

    @pytest.fixture
    def _connect_end_to_(self, request):
        return method_mock(request, Connector, "_connect_end_to", autospec=True)

    @pytest.fixture
    def _move_begin_to_cxn_(self, request):
        return method_mock(request, Connector, "_move_begin_to_cxn", autospec=True)

    @pytest.fixture
    def _move_end_to_cxn_(self, request):
        return method_mock(request, Connector, "_move_end_to_cxn", autospec=True)

    @pytest.fixture
    def shape_(self, request):
        return instance_mock(request, BaseShape)



# encoding: utf-8

"""Test suite for pptx.slide module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.dml.fill import FillFormat
from pptx.enum.shapes import PP_PLACEHOLDER
from pptx.package import Package
from pptx.parts.presentation import PresentationPart
from pptx.parts.slide import SlideLayoutPart, SlideMasterPart, SlidePart
from pptx.presentation import Presentation
from pptx.shapes.base import BaseShape
from pptx.shapes.placeholder import LayoutPlaceholder, NotesSlidePlaceholder
from pptx.shapes.shapetree import (
    LayoutPlaceholders,
    LayoutShapes,
    MasterPlaceholders,
    MasterShapes,
    NotesSlidePlaceholders,
    NotesSlideShapes,
    SlidePlaceholders,
    SlideShapes,
)
from pptx.slide import (
    _Background,
    _BaseMaster,
    _BaseSlide,
    NotesMaster,
    NotesSlide,
    Slide,
    SlideLayout,
    SlideLayouts,
    SlideMaster,
    SlideMasters,
    Slides,
)
from pptx.text.text import TextFrame

from .unitutil.cxml import element, xml
from .unitutil.mock import call, class_mock, instance_mock, method_mock, property_mock


class Describe_BaseSlide(object):
    def it_knows_its_name(self, name_get_fixture):
        base_slide, expected_value = name_get_fixture
        assert base_slide.name == expected_value

    def it_can_change_its_name(self, name_set_fixture):
        base_slide, new_value, expected_xml = name_set_fixture
        base_slide.name = new_value
        assert base_slide._element.xml == expected_xml

    def it_provides_access_to_its_background(self, background_fixture):
        slide, _Background_, cSld, background_ = background_fixture

        background = slide.background

        _Background_.assert_called_once_with(cSld)
        assert background is background_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def background_fixture(self, _Background_, background_):
        sld = element("p:sld/p:cSld")
        slide = _BaseSlide(sld, None)
        cSld = sld.xpath("//p:cSld")[0]
        _Background_.return_value = background_
        return slide, _Background_, cSld, background_

    @pytest.fixture(
        params=[("p:sld/p:cSld", ""), ("p:sld/p:cSld{name=Foobar}", "Foobar")]
    )
    def name_get_fixture(self, request):
        sld_cxml, expected_name = request.param
        base_slide = _BaseSlide(element(sld_cxml), None)
        return base_slide, expected_name

    @pytest.fixture(
        params=[
            ("p:sld/p:cSld", "foo", "p:sld/p:cSld{name=foo}"),
            ("p:sld/p:cSld{name=foo}", "bar", "p:sld/p:cSld{name=bar}"),
            ("p:sld/p:cSld{name=bar}", "", "p:sld/p:cSld"),
            ("p:sld/p:cSld{name=bar}", None, "p:sld/p:cSld"),
            ("p:sld/p:cSld", "", "p:sld/p:cSld"),
            ("p:sld/p:cSld", None, "p:sld/p:cSld"),
        ]
    )
    def name_set_fixture(self, request):
        xSld_cxml, new_value, expected_cxml = request.param
        base_slide = _BaseSlide(element(xSld_cxml), None)
        expected_xml = xml(expected_cxml)
        return base_slide, new_value, expected_xml

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _Background_(self, request):
        return class_mock(request, "pptx.slide._Background")

    @pytest.fixture
    def background_(self, request):
        return instance_mock(request, _Background)


class Describe_BaseMaster(object):
    def it_is_a_BaseSlide_subclass(self, subclass_fixture):
        base_master = subclass_fixture
        assert isinstance(base_master, _BaseSlide)

    def it_provides_access_to_its_placeholders(self, placeholders_fixture):
        base_master, MasterPlaceholders_, spTree, placeholders_ = placeholders_fixture
        placeholders = base_master.placeholders
        MasterPlaceholders_.assert_called_once_with(spTree, base_master)
        assert placeholders is placeholders_

    def it_provides_access_to_its_shapes(self, shapes_fixture):
        base_master, MasterShapes_, spTree, shapes_ = shapes_fixture
        shapes = base_master.shapes
        MasterShapes_.assert_called_once_with(spTree, base_master)
        assert shapes is shapes_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def placeholders_fixture(self, MasterPlaceholders_, placeholders_):
        master_element = element("p:sldMaster/p:cSld/p:spTree")
        base_master = _BaseMaster(master_element, None)
        spTree = master_element.xpath("//p:spTree")[0]
        return base_master, MasterPlaceholders_, spTree, placeholders_

    @pytest.fixture
    def shapes_fixture(self, MasterShapes_, shapes_):
        master_element = element("p:sldMaster/p:cSld/p:spTree")
        base_master = _BaseMaster(master_element, None)
        spTree = master_element.xpath("//p:spTree")[0]
        return base_master, MasterShapes_, spTree, shapes_

    @pytest.fixture
    def subclass_fixture(self):
        return _BaseMaster(None, None)

    # fixture components -----------------------------------

    @pytest.fixture
    def MasterPlaceholders_(self, request, placeholders_):
        return class_mock(
            request, "pptx.slide.MasterPlaceholders", return_value=placeholders_
        )

    @pytest.fixture
    def MasterShapes_(self, request, shapes_):
        return class_mock(request, "pptx.slide.MasterShapes", return_value=shapes_)

    @pytest.fixture
    def placeholders_(self, request):
        return instance_mock(request, MasterPlaceholders)

    @pytest.fixture
    def shapes_(self, request):
        return instance_mock(request, MasterShapes)


class DescribeNotesSlide(object):
    def it_can_clone_the_notes_master_placeholders(self, clone_fixture):
        notes_slide, notes_master_, clone_placeholder_, calls = clone_fixture
        notes_slide.clone_master_placeholders(notes_master_)
        assert clone_placeholder_.call_args_list == calls

    def it_provides_access_to_its_shapes(self, shapes_fixture):
        notes_slide, NotesSlideShapes_, spTree, shapes_ = shapes_fixture
        shapes = notes_slide.shapes
        NotesSlideShapes_.assert_called_once_with(spTree, notes_slide)
        assert shapes is shapes_

    def it_provides_access_to_its_placeholders(self, placeholders_fixture):
        notes_slide, NotesSlidePlaceholders_, spTree, placeholders_ = (
            placeholders_fixture
        )
        placeholders = notes_slide.placeholders
        NotesSlidePlaceholders_.assert_called_once_with(spTree, notes_slide)
        assert placeholders is placeholders_

    def it_provides_access_to_its_notes_placeholder(self, notes_ph_fixture):
        notes_slide, expected_value = notes_ph_fixture
        placeholder = notes_slide.notes_placeholder
        assert placeholder is expected_value

    def it_provides_access_to_its_notes_text_frame(self, notes_tf_fixture):
        notes_slide, expected_value = notes_tf_fixture
        text_frame = notes_slide.notes_text_frame
        assert text_frame is expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def clone_fixture(self, notes_master_, clone_placeholder_, shapes_prop_, shapes_):
        notes_slide = NotesSlide(None, None)
        placeholders = notes_master_.placeholders = (
            BaseShape(element("p:sp/p:nvSpPr/p:nvPr/p:ph{type=body}"), None),
            BaseShape(element("p:sp/p:nvSpPr/p:nvPr/p:ph{type=dt}"), None),
        )
        calls = [call(placeholders[0])]
        shapes_.clone_placeholder = clone_placeholder_
        return notes_slide, notes_master_, clone_placeholder_, calls

    @pytest.fixture(
        params=[
            (("SLIDE_IMAGE", "BODY", "SLIDE_NUMBER"), 1),
            (("DATE", "SLIDE_IMAGE", "FOOTER"), None),
        ]
    )
    def notes_ph_fixture(self, request, placeholders_prop_):
        type_names, match = request.param
        notes_slide = NotesSlide(None, None)
        placeholders_ = []
        for type_name in type_names:
            placeholder_ = instance_mock(
                request, NotesSlidePlaceholder, name="%s-placeholder" % type_name
            )
            placeholder_.placeholder_format.type = getattr(PP_PLACEHOLDER, type_name)
            placeholders_.append(placeholder_)
        placeholders_prop_.return_value = placeholders_
        expected_value = None if match is None else placeholders_[match]
        return notes_slide, expected_value

    @pytest.fixture(params=[True, False])
    def notes_tf_fixture(
        self, request, notes_placeholder_prop_, placeholder_, text_frame_
    ):
        has_text_frame = request.param
        notes_slide = NotesSlide(None, None)
        if has_text_frame:
            notes_placeholder_prop_.return_value = placeholder_
            placeholder_.text_frame = text_frame_
            expected_value = text_frame_
        else:
            notes_placeholder_prop_.return_value = None
            expected_value = None
        return notes_slide, expected_value

    @pytest.fixture
    def placeholders_fixture(self, NotesSlidePlaceholders_, placeholders_):
        notes = element("p:notes/p:cSld/p:spTree")
        notes_slide = NotesSlide(notes, None)
        spTree = notes.xpath("//p:spTree")[0]
        return notes_slide, NotesSlidePlaceholders_, spTree, placeholders_

    @pytest.fixture
    def shapes_fixture(self, NotesSlideShapes_, shapes_):
        notes = element("p:notes/p:cSld/p:spTree")
        notes_slide = NotesSlide(notes, None)
        spTree = notes.xpath("//p:spTree")[0]
        return notes_slide, NotesSlideShapes_, spTree, shapes_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def clone_placeholder_(self, request):
        return method_mock(request, NotesSlideShapes, "clone_placeholder")

    @pytest.fixture
    def notes_master_(self, request):
        return instance_mock(request, NotesMaster)

    @pytest.fixture
    def notes_placeholder_prop_(self, request, placeholder_):
        return property_mock(
            request, NotesSlide, "notes_placeholder", return_value=placeholder_
        )

    @pytest.fixture
    def NotesSlidePlaceholders_(self, request, placeholders_):
        return class_mock(
            request, "pptx.slide.NotesSlidePlaceholders", return_value=placeholders_
        )

    @pytest.fixture
    def NotesSlideShapes_(self, request, shapes_):
        return class_mock(request, "pptx.slide.NotesSlideShapes", return_value=shapes_)

    @pytest.fixture
    def placeholder_(self, request):
        return instance_mock(request, NotesSlidePlaceholder)

    @pytest.fixture
    def placeholders_(self, request):
        return instance_mock(request, NotesSlidePlaceholders)

    @pytest.fixture
    def placeholders_prop_(self, request, placeholders_):
        return property_mock(
            request, NotesSlide, "placeholders", return_value=placeholders_
        )

    @pytest.fixture
    def shapes_(self, request):
        return instance_mock(request, NotesSlideShapes)

    @pytest.fixture
    def shapes_prop_(self, request, shapes_):
        return property_mock(request, NotesSlide, "shapes", return_value=shapes_)

    @pytest.fixture
    def text_frame_(self, request):
        return instance_mock(request, TextFrame)


class DescribeSlide(object):
    def it_is_a_BaseSlide_subclass(self, subclass_fixture):
        slide = subclass_fixture
        assert isinstance(slide, _BaseSlide)

    def it_provides_access_to_its_background(self, background_fixture):
        slide, _BaseSlide_background_, background_ = background_fixture

        background = slide.background

        _BaseSlide_background_.assert_called_once_with()
        assert background is background_

    def it_knows_whether_it_follows_the_mstr_bkgd(self, follow_get_fixture):
        slide, expected_value = follow_get_fixture
        follows = slide.follow_master_background
        assert follows is expected_value

    def it_knows_whether_it_has_a_notes_slide(self, has_notes_slide_fixture):
        slide, expected_value = has_notes_slide_fixture
        assert slide.has_notes_slide is expected_value

    def it_knows_its_slide_id(self, slide_id_fixture):
        slide, expected_value = slide_id_fixture
        assert slide.slide_id == expected_value

    def it_provides_access_to_its_shapes(self, shapes_fixture):
        slide, SlideShapes_, spTree, shapes_ = shapes_fixture
        shapes = slide.shapes
        SlideShapes_.assert_called_once_with(spTree, slide)
        assert shapes is shapes_

    def it_provides_access_to_its_placeholders(self, placeholders_fixture):
        slide, SlidePlaceholders_, spTree, placeholders_ = placeholders_fixture
        placeholders = slide.placeholders
        SlidePlaceholders_.assert_called_once_with(spTree, slide)
        assert placeholders is placeholders_

    def it_provides_access_to_its_slide_layout(self, layout_fixture):
        slide, slide_layout_ = layout_fixture
        assert slide.slide_layout is slide_layout_

    def it_provides_access_to_its_notes_slide(self, notes_slide_fixture):
        slide, notes_slide_ = notes_slide_fixture
        assert slide.notes_slide is notes_slide_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def background_fixture(self, _BaseSlide_background_, background_):
        slide = Slide(None, None)
        _BaseSlide_background_.return_value = background_
        return slide, _BaseSlide_background_, background_

    @pytest.fixture(params=[("p:sld/p:cSld", True), ("p:sld/p:cSld/p:bg", False)])
    def follow_get_fixture(self, request):
        pSld_cxml, expected_value = request.param
        slide = Slide(element(pSld_cxml), None)
        return slide, expected_value

    @pytest.fixture
    def has_notes_slide_fixture(self, part_prop_, slide_part_):
        slide = Slide(None, None)
        expected_value = slide_part_.has_notes_slide = 42
        return slide, expected_value

    @pytest.fixture
    def layout_fixture(self, slide_layout_, part_prop_, slide_part_):
        slide = Slide(None, None)
        part_prop_.return_value = slide_part_
        slide_part_.slide_layout = slide_layout_
        return slide, slide_layout_

    @pytest.fixture
    def notes_slide_fixture(self, notes_slide_, part_prop_, slide_part_):
        slide = Slide(None, None)
        slide_part_.notes_slide = notes_slide_
        return slide, notes_slide_

    @pytest.fixture
    def placeholders_fixture(self, SlidePlaceholders_, placeholders_):
        sld = element("p:sld/p:cSld/p:spTree")
        slide = Slide(sld, None)
        spTree = sld.xpath("//p:spTree")[0]
        return slide, SlidePlaceholders_, spTree, placeholders_

    @pytest.fixture
    def shapes_fixture(self, SlideShapes_, shapes_):
        sld = element("p:sld/p:cSld/p:spTree")
        spTree = sld.xpath("//p:spTree")[0]
        slide = Slide(sld, None)
        return slide, SlideShapes_, spTree, shapes_

    @pytest.fixture
    def slide_id_fixture(self, part_prop_, slide_part_):
        slide = Slide(None, None)
        slide_id = 256
        slide_part_.slide_id = slide_id
        return slide, slide_id

    @pytest.fixture
    def subclass_fixture(self):
        return Slide(None, None)

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _BaseSlide_background_(self, request):
        return property_mock(request, _BaseSlide, "background")

    @pytest.fixture
    def background_(self, request):
        return instance_mock(request, _Background)

    @pytest.fixture
    def notes_slide_(self, request):
        return instance_mock(request, NotesSlide)

    @pytest.fixture
    def part_prop_(self, request, slide_part_):
        return property_mock(request, Slide, "part", return_value=slide_part_)

    @pytest.fixture
    def placeholders_(self, request):
        return instance_mock(request, SlidePlaceholders)

    @pytest.fixture
    def SlidePlaceholders_(self, request, placeholders_):
        return class_mock(
            request, "pptx.slide.SlidePlaceholders", return_value=placeholders_
        )

    @pytest.fixture
    def SlideShapes_(self, request, shapes_):
        return class_mock(request, "pptx.slide.SlideShapes", return_value=shapes_)

    @pytest.fixture
    def shapes_(self, request):
        return instance_mock(request, SlideShapes)

    @pytest.fixture
    def slide_layout_(self, request):
        return instance_mock(request, SlideLayout)

    @pytest.fixture
    def slide_part_(self, request):
        return instance_mock(request, SlidePart)


class DescribeSlides(object):
    def it_supports_indexed_access(self, getitem_fixture):
        slides, prs_part_, rId, slide_ = getitem_fixture
        slide = slides[0]
        prs_part_.related_slide.assert_called_once_with(rId)
        assert slide is slide_

    def it_raises_on_slide_index_out_of_range(self, getitem_raises_fixture):
        slides = getitem_raises_fixture
        with pytest.raises(IndexError):
            slides[2]

    def it_knows_the_index_of_a_slide_it_contains(self, index_fixture):
        slides, slide, expected_value = index_fixture
        index = slides.index(slide)
        assert index == expected_value

    def it_raises_on_slide_not_in_collection(self, raises_fixture):
        slides, slide = raises_fixture
        with pytest.raises(ValueError):
            slides.index(slide)

    def it_can_iterate_its_slides(self, iter_fixture):
        slides, related_slide_, calls, expected_value = iter_fixture
        slide_lst = [s for s in slides]
        assert related_slide_.call_args_list == calls
        assert slide_lst == expected_value

    def it_supports_len(self, len_fixture):
        slides, expected_value = len_fixture
        assert len(slides) == expected_value

    def it_can_add_a_new_slide(self, add_fixture):
        slides, slide_layout_, part_ = add_fixture[:3]
        clone_layout_placeholders_, expected_xml, slide_ = add_fixture[3:]

        slide = slides.add_slide(slide_layout_)

        part_.add_slide.assert_called_once_with(slide_layout_)
        clone_layout_placeholders_.assert_called_once_with(slide_layout_)
        assert slides._sldIdLst.xml == expected_xml
        assert slide is slide_

    def it_finds_a_slide_by_slide_id(self, get_fixture):
        slides, slide_id, default, prs_part_, expected_value = get_fixture
        slide = slides.get(slide_id, default)
        prs_part_.get_slide.assert_called_once_with(slide_id)
        assert slide is expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def add_fixture(self, slide_layout_, part_prop_, slide_):
        slides = Slides(element("p:sldIdLst/p:sldId{r:id=rId1}"), None)
        part_ = part_prop_.return_value
        clone_layout_placeholders_ = slide_.shapes.clone_layout_placeholders
        expected_xml = xml("p:sldIdLst/(p:sldId{r:id=rId1},p:sldId{r:id=rId2,id=256})")
        part_.add_slide.return_value = "rId2", slide_
        return (
            slides,
            slide_layout_,
            part_,
            clone_layout_placeholders_,
            expected_xml,
            slide_,
        )

    @pytest.fixture(params=[True, False])
    def get_fixture(self, request, part_prop_, prs_part_, slide_):
        found = request.param
        slides = Slides(None, None)
        slide_id, default = 256, "foobar"
        expected_value = slide_ if found else default
        prs_part_.get_slide.return_value = slide_ if found else None
        return slides, slide_id, default, prs_part_, expected_value

    @pytest.fixture
    def getitem_fixture(self, prs_part_, slide_, part_prop_):
        sldIdLst = element("p:sldIdLst/p:sldId{r:id=rId1}")
        slides = Slides(sldIdLst, None)
        prs_part_.related_slide.return_value = slide_
        return slides, prs_part_, "rId1", slide_

    @pytest.fixture
    def getitem_raises_fixture(self):
        sldIdLst = element("p:sldIdLst/p:sldId{r:id=rId1}")
        slides = Slides(sldIdLst, None)
        return slides

    @pytest.fixture(params=[0, 1])
    def index_fixture(self, request, part_prop_):
        idx = request.param
        sldIdLst = element("p:sldIdLst/(p:sldId{r:id=a},p:sldId{r:id=b})")
        slides = Slides(sldIdLst, None)
        _slides = [Slide(element("p:sld"), None), Slide(element("p:sld"), None)]
        part_prop_.return_value.related_slide.side_effect = _slides
        return slides, _slides[idx], idx

    @pytest.fixture
    def iter_fixture(self, part_prop_, slide_):
        sldIdLst = element("p:sldIdLst/(p:sldId{r:id=a},p:sldId{r:id=b})")
        slides = Slides(sldIdLst, None)
        related_slide_ = part_prop_.return_value.related_slide
        related_slide_.return_value = slide_
        calls = [call("a"), call("b")]
        _slides = [slide_, slide_]
        return slides, related_slide_, calls, _slides

    @pytest.fixture(
        params=[
            ("p:sldIdLst", 0),
            ("p:sldIdLst/p:sldId{r:id=a}", 1),
            ("p:sldIdLst/(p:sldId{r:id=a},p:sldId{r:id=b})", 2),
        ]
    )
    def len_fixture(self, request):
        sldIdLst_cxml, expected_value = request.param
        slides = Slides(element(sldIdLst_cxml), None)
        return slides, expected_value

    @pytest.fixture
    def raises_fixture(self):
        slides = Slides(element("p:sldIdLst"), None)
        slide = Slide(element("p:sld"), None)
        return slides, slide

    # fixture components ---------------------------------------------

    @pytest.fixture
    def part_prop_(self, request, prs_part_):
        return property_mock(request, Slides, "part", return_value=prs_part_)

    @pytest.fixture
    def prs_part_(self, request):
        return instance_mock(request, PresentationPart)

    @pytest.fixture
    def slide_(self, request):
        return instance_mock(request, Slide)

    @pytest.fixture
    def slide_layout_(self, request):
        return instance_mock(request, SlideLayout)


class DescribeSlideLayout(object):
    def it_is_a_BaseSlide_subclass(self):
        slide_layout = SlideLayout(None, None)
        assert isinstance(slide_layout, _BaseSlide)

    def it_can_iterate_its_clonable_placeholders(self, cloneable_fixture):
        slide_layout, expected_placeholders = cloneable_fixture
        cloneable = list(slide_layout.iter_cloneable_placeholders())
        assert cloneable == expected_placeholders

    def it_provides_access_to_its_placeholders(
        self, LayoutPlaceholders_, placeholders_
    ):
        sldLayout = element("p:sldLayout/p:cSld/p:spTree")
        spTree = sldLayout.xpath("//p:spTree")[0]
        slide_layout = SlideLayout(sldLayout, None)

        placeholders = slide_layout.placeholders

        LayoutPlaceholders_.assert_called_once_with(spTree, slide_layout)
        assert placeholders is placeholders_

    def it_provides_access_to_its_shapes(self, LayoutShapes_, shapes_):
        sldLayout = element("p:sldLayout/p:cSld/p:spTree")
        spTree = sldLayout.xpath("//p:spTree")[0]
        slide_layout = SlideLayout(sldLayout, None)

        shapes = slide_layout.shapes

        LayoutShapes_.assert_called_once_with(spTree, slide_layout)
        assert shapes is shapes_

    def it_provides_access_to_its_slide_master(self, slide_master_, part_prop_):
        part_prop_.return_value.slide_master = slide_master_
        slide_layout = SlideLayout(None, None)

        slide_master = slide_layout.slide_master

        assert slide_master is slide_master_

    def it_knows_which_slides_are_based_on_it(
        self,
        used_by_fixture,
        part_prop_,
        slide_layout_part_,
        package_,
        presentation_part_,
        presentation_,
    ):
        presentation_, slide_layout, expected_value = used_by_fixture
        part_prop_.return_value = slide_layout_part_
        slide_layout_part_.package = package_
        package_.presentation_part = presentation_part_
        presentation_part_.presentation = presentation_

        used_by_slides = slide_layout.used_by_slides

        assert used_by_slides == expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ((PP_PLACEHOLDER.TITLE, PP_PLACEHOLDER.BODY), (0, 1)),
            ((PP_PLACEHOLDER.TITLE, PP_PLACEHOLDER.DATE), (0,)),
            ((PP_PLACEHOLDER.FOOTER, PP_PLACEHOLDER.OBJECT), (1,)),
            ((PP_PLACEHOLDER.SLIDE_NUMBER, PP_PLACEHOLDER.FOOTER), ()),
        ]
    )
    def cloneable_fixture(
        self, request, placeholders_prop_, placeholder_, placeholder_2_
    ):
        ph_types, expected_indices = request.param
        slide_layout = SlideLayout(None, None)
        placeholder_.element.ph_type = ph_types[0]
        placeholder_2_.element.ph_type = ph_types[1]
        _placeholders = (placeholder_, placeholder_2_)
        expected_placeholders = [_placeholders[idx] for idx in expected_indices]
        placeholders_prop_.return_value = _placeholders
        return slide_layout, expected_placeholders

    @pytest.fixture(params=[(), (0,), (1,), (0, 1)])
    def used_by_fixture(self, request, presentation_, slide_, slide_2_):
        used_by_idxs = request.param
        slides = (slide_, slide_2_)
        slide_layout = SlideLayout(None, None)
        for idx, s in enumerate(slides):
            s.slide_layout = slide_layout if idx in used_by_idxs else None
        presentation_.slides = slides
        expected_value = tuple(s for i, s in enumerate(slides) if i in used_by_idxs)
        return presentation_, slide_layout, expected_value

    # fixture components -----------------------------------

    @pytest.fixture
    def LayoutPlaceholders_(self, request, placeholders_):
        return class_mock(
            request, "pptx.slide.LayoutPlaceholders", return_value=placeholders_
        )

    @pytest.fixture
    def LayoutShapes_(self, request, shapes_):
        return class_mock(request, "pptx.slide.LayoutShapes", return_value=shapes_)

    @pytest.fixture
    def package_(self, request):
        return instance_mock(request, Package)

    @pytest.fixture
    def part_prop_(self, request, slide_layout_part_):
        return property_mock(
            request, SlideLayout, "part", return_value=slide_layout_part_
        )

    @pytest.fixture
    def placeholder_(self, request):
        return instance_mock(request, LayoutPlaceholder)

    @pytest.fixture
    def placeholder_2_(self, request):
        return instance_mock(request, LayoutPlaceholder)

    @pytest.fixture
    def placeholders_(self, request):
        return instance_mock(request, LayoutPlaceholders)

    @pytest.fixture
    def placeholders_prop_(self, request, placeholders_):
        return property_mock(
            request, SlideLayout, "placeholders", return_value=placeholders_
        )

    @pytest.fixture
    def presentation_(self, request):
        return instance_mock(request, Presentation)

    @pytest.fixture
    def presentation_part_(self, request):
        return instance_mock(request, PresentationPart)

    @pytest.fixture
    def shapes_(self, request):
        return instance_mock(request, LayoutShapes)

    @pytest.fixture
    def slide_(self, request):
        return instance_mock(request, Slide)

    @pytest.fixture
    def slide_2_(self, request):
        return instance_mock(request, Slide)

    @pytest.fixture
    def slide_layout_part_(self, request):
        return instance_mock(request, SlideLayoutPart)

    @pytest.fixture
    def slide_master_(self, request):
        return instance_mock(request, SlideMaster)


class DescribeSlideLayouts(object):
    def it_supports_len(self, len_fixture):
        slide_layouts, expected_value = len_fixture
        assert len(slide_layouts) == expected_value

    def it_can_iterate_its_slide_layouts(self, part_prop_, slide_master_part_):
        sldLayoutIdLst = element(
            "p:sldLayoutIdLst/(p:sldLayoutId{r:id=a},p:sldLayoutId{r:id=b})"
        )
        _slide_layouts = [
            SlideLayout(element("p:sldLayout"), None),
            SlideLayout(element("p:sldLayout"), None),
        ]
        part_prop_.return_value = slide_master_part_
        related_slide_layout_ = slide_master_part_.related_slide_layout
        related_slide_layout_.side_effect = _slide_layouts
        slide_layouts = SlideLayouts(sldLayoutIdLst, None)

        slide_layout_lst = [sl for sl in slide_layouts]

        assert related_slide_layout_.call_args_list == [call("a"), call("b")]
        assert slide_layout_lst == _slide_layouts

    def it_supports_indexed_access(self, slide_layout_, part_prop_, slide_master_part_):
        part_prop_.return_value = slide_master_part_
        slide_master_part_.related_slide_layout.return_value = slide_layout_
        slide_layouts = SlideLayouts(
            element("p:sldLayoutIdLst/p:sldLayoutId{r:id=rId1}"), None
        )

        slide_layout = slide_layouts[0]

        slide_master_part_.related_slide_layout.assert_called_once_with("rId1")
        assert slide_layout is slide_layout_

    def but_it_raises_on_index_out_of_range(self, part_prop_):
        slide_layouts = SlideLayouts(
            element("p:sldLayoutIdLst/p:sldLayoutId{r:id=rId1}"), None
        )
        with pytest.raises(IndexError):
            slide_layouts[1]

    def it_can_find_a_slide_layout_by_name(
        self, _iter_, slide_layout_, slide_layout_2_
    ):
        _iter_.return_value = iter((slide_layout_, slide_layout_2_))
        slide_layout_2_.name = "pick me!"
        slide_layouts = SlideLayouts(None, None)

        slide_layout = slide_layouts.get_by_name("pick me!")

        assert slide_layout is slide_layout_2_

    def but_it_returns_the_default_value_when_no_layout_has_that_name(
        self, _iter_, slide_layout_, slide_layout_2_
    ):
        _iter_.return_value = iter((slide_layout_, slide_layout_2_))
        slide_layout_2_.name = "not the droid you're looking for"
        slide_layouts = SlideLayouts(None, None)

        # ---default-default is None---
        slide_layout = slide_layouts.get_by_name("pick me!")
        assert slide_layout is None

        # ---but default can be specified---
        slide_layout = slide_layouts.get_by_name("pick me!", "default-value")
        assert slide_layout == "default-value"

    def it_knows_the_index_of_each_of_its_slide_layouts(
        self, _iter_, slide_layout_, slide_layout_2_
    ):
        _iter_.return_value = iter((slide_layout_, slide_layout_2_))
        slide_layouts = SlideLayouts(None, None)

        index = slide_layouts.index(slide_layout_2_)

        assert index == 1

    def but_it_raises_on_slide_layout_not_in_collection(
        self, _iter_, slide_layout_, slide_layout_2_
    ):
        _iter_.return_value = iter((slide_layout_,))
        slide_layouts = SlideLayouts(None, None)

        with pytest.raises(ValueError) as e:
            slide_layouts.index(slide_layout_2_)
        assert str(e.value) == "layout not in this SlideLayouts collection"

    def it_can_remove_an_unused_slide_layout(
        self, slide_layout_, index_, slide_master_, slide_master_part_
    ):
        slide_layout_.used_by_slides = ()
        index_.return_value = 0
        sldLayoutIdLst = element(
            "p:sldLayoutIdLst/(p:sldLayoutId{r:id=rId1},p:sldLayoutId{r:id=rId2})"
        )
        slide_layout_.slide_master = slide_master_
        slide_master_.part = slide_master_part_
        slide_layouts = SlideLayouts(sldLayoutIdLst, None)

        slide_layouts.remove(slide_layout_)

        assert slide_layouts._sldLayoutIdLst.xml == xml(
            "p:sldLayoutIdLst/p:sldLayoutId{r:id=rId2}"
        )
        slide_master_part_.drop_rel.assert_called_once_with("rId1")

    def but_it_raises_on_attempt_to_remove_slide_layout_in_use(
        self, slide_layout_, slide_
    ):
        slide_layout_.used_by_slides = (slide_,)
        slide_layouts = SlideLayouts(None, None)

        with pytest.raises(ValueError):
            slide_layouts.remove(slide_layout_)

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("p:sldLayoutIdLst", 0),
            ("p:sldLayoutIdLst/p:sldLayoutId", 1),
            ("p:sldLayoutIdLst/(p:sldLayoutId,p:sldLayoutId)", 2),
        ]
    )
    def len_fixture(self, request):
        sldLayoutIdLst_cxml, expected_value = request.param
        slide_layouts = SlideLayouts(element(sldLayoutIdLst_cxml), None)
        return slide_layouts, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def index_(self, request):
        return method_mock(request, SlideLayouts, "index")

    @pytest.fixture
    def _iter_(self, request):
        return method_mock(request, SlideLayouts, "__iter__")

    @pytest.fixture
    def part_prop_(self, request):
        return property_mock(request, SlideLayouts, "part")

    @pytest.fixture
    def slide_(self, request):
        return instance_mock(request, Slide)

    @pytest.fixture
    def slide_layout_(self, request):
        return instance_mock(request, SlideLayout)

    @pytest.fixture
    def slide_layout_2_(self, request):
        return instance_mock(request, SlideLayout)

    @pytest.fixture
    def slide_master_(self, request):
        return instance_mock(request, SlideMaster)

    @pytest.fixture
    def slide_master_part_(self, request):
        return instance_mock(request, SlideMasterPart)


class DescribeSlideMaster(object):
    def it_is_a_BaseMaster_subclass(self, subclass_fixture):
        slide_master = subclass_fixture
        assert isinstance(slide_master, _BaseMaster)

    def it_provides_access_to_its_slide_layouts(self, layouts_fixture):
        slide_master, SlideLayouts_, sldLayoutIdLst, slide_layouts_ = layouts_fixture
        slide_layouts = slide_master.slide_layouts
        SlideLayouts_.assert_called_once_with(sldLayoutIdLst, slide_master)
        assert slide_layouts is slide_layouts_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def layouts_fixture(self, SlideLayouts_, slide_layouts_):
        sldMaster = element("p:sldMaster/p:sldLayoutIdLst")
        slide_master = SlideMaster(sldMaster, None)
        sldMasterIdLst = sldMaster.sldLayoutIdLst
        return slide_master, SlideLayouts_, sldMasterIdLst, slide_layouts_

    @pytest.fixture
    def subclass_fixture(self):
        return SlideMaster(None, None)

    # fixture components -----------------------------------

    @pytest.fixture
    def SlideLayouts_(self, request, slide_layouts_):
        return class_mock(
            request, "pptx.slide.SlideLayouts", return_value=slide_layouts_
        )

    @pytest.fixture
    def slide_layouts_(self, request):
        return instance_mock(request, SlideLayouts)


class DescribeSlideMasters(object):
    def it_knows_how_many_masters_it_contains(self, len_fixture):
        slide_masters, expected_value = len_fixture
        assert len(slide_masters) == expected_value

    def it_can_iterate_the_slide_masters(self, iter_fixture):
        slide_masters, related_slide_master_, calls, expected_values = iter_fixture
        _slide_masters = [sm for sm in slide_masters]
        assert related_slide_master_.call_args_list == calls
        assert _slide_masters == expected_values

    def it_supports_indexed_access(self, getitem_fixture):
        slide_masters, part_, slide_master_, rId = getitem_fixture
        slide_master = slide_masters[0]
        part_.related_slide_master.assert_called_once_with(rId)
        assert slide_master is slide_master_

    def it_raises_on_index_out_of_range(self, getitem_raises_fixture):
        slides = getitem_raises_fixture
        with pytest.raises(IndexError):
            slides[1]

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def getitem_fixture(self, part_, slide_master_, part_prop_):
        slide_masters = SlideMasters(
            element("p:sldMasterIdLst/p:sldMasterId{r:id=rId1}"), None
        )
        part_.related_slide_master.return_value = slide_master_
        return slide_masters, part_, slide_master_, "rId1"

    @pytest.fixture
    def getitem_raises_fixture(self, part_prop_):
        return SlideMasters(element("p:sldMasterIdLst/p:sldMasterId{r:id=rId1}"), None)

    @pytest.fixture
    def iter_fixture(self, part_prop_):
        sldMasterIdLst = element(
            "p:sldMasterIdLst/(p:sldMasterId{r:id=a},p:sldMasterId{r:id=b})"
        )
        slide_masters = SlideMasters(sldMasterIdLst, None)
        related_slide_master_ = part_prop_.return_value.related_slide_master
        calls = [call("a"), call("b")]
        _slide_masters = [
            SlideMaster(element("p:sldMaster"), None),
            SlideMaster(element("p:sldMaster"), None),
        ]
        related_slide_master_.side_effect = _slide_masters
        return slide_masters, related_slide_master_, calls, _slide_masters

    @pytest.fixture(
        params=[
            ("p:sldMasterIdLst", 0),
            ("p:sldMasterIdLst/p:sldMasterId", 1),
            ("p:sldMasterIdLst/(p:sldMasterId,p:sldMasterId)", 2),
        ]
    )
    def len_fixture(self, request):
        sldMasterIdLst_cxml, expected_value = request.param
        slide_masters = SlideMasters(element(sldMasterIdLst_cxml), None)
        return slide_masters, expected_value

    # fixture components ---------------------------------------------

    @pytest.fixture
    def part_(self, request):
        return instance_mock(request, PresentationPart)

    @pytest.fixture
    def part_prop_(self, request, part_):
        return property_mock(request, SlideMasters, "part", return_value=part_)

    @pytest.fixture
    def slide_master_(self, request):
        return instance_mock(request, SlideMaster)


class Describe_Background(object):
    def it_provides_access_to_its_fill(self, fill_fixture):
        background, cSld, expected_xml = fill_fixture[:3]
        from_fill_parent_, fill_ = fill_fixture[3:]

        fill = background.fill

        assert cSld.xml == expected_xml
        from_fill_parent_.assert_called_once_with(cSld.xpath("p:bg/p:bgPr")[0])
        assert fill is fill_

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("p:cSld{a:b=c}", "p:cSld{a:b=c}/p:bg/p:bgPr/(a:noFill,a:effectLst)"),
            (
                "p:cSld{a:b=c}/p:bg/p:bgRef",
                "p:cSld{a:b=c}/p:bg/p:bgPr/(a:noFill,a:effectLst)",
            ),
            ("p:cSld/p:bg/p:bgPr/a:solidFill", "p:cSld/p:bg/p:bgPr/a:solidFill"),
        ]
    )
    def fill_fixture(self, request, from_fill_parent_, fill_):
        cSld_xml, expected_cxml = request.param
        cSld = element(cSld_xml)
        background = _Background(cSld)

        from_fill_parent_.return_value = fill_

        expected_xml = xml(expected_cxml)
        return background, cSld, expected_xml, from_fill_parent_, fill_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def fill_(self, request):
        return instance_mock(request, FillFormat)

    @pytest.fixture
    def from_fill_parent_(self, request):
        return method_mock(request, FillFormat, "from_fill_parent")



# encoding: utf-8

"""Visual effects on a shape such as shadow, glow, and reflection."""

from __future__ import absolute_import, division, print_function, unicode_literals


class ShadowFormat(object):
    """Provides access to shadow effect on a shape."""

    def __init__(self, spPr):
        # ---spPr may also be a grpSpPr; both have a:effectLst child---
        self._element = spPr

    @property
    def inherit(self):
        """True if shape inherits shadow settings.

        Read/write. An explicitly-defined shadow setting on a shape causes
        this property to return |False|. A shape with no explicitly-defined
        shadow setting inherits its shadow settings from the style hierarchy
        (and so returns |True|).

        Assigning |True| causes any explicitly-defined shadow setting to be
        removed and inheritance is restored. Note this has the side-effect of
        removing **all** explicitly-defined effects, such as glow and
        reflection, and restoring inheritance for all effects on the shape.
        Assigning |False| causes the inheritance link to be broken and **no**
        effects to appear on the shape.
        """
        if self._element.effectLst is None:
            return True
        return False

    @inherit.setter
    def inherit(self, value):
        inherit = bool(value)
        if inherit:
            # ---remove any explicitly-defined effects
            self._element._remove_effectLst()
        else:
            # ---ensure at least the effectLst element is present
            self._element.get_or_add_effectLst()



# encoding: utf-8

"""Gherkin step implementations for slide-related features."""

from __future__ import absolute_import, division, print_function, unicode_literals

from behave import given, then

from pptx import Presentation

from helpers import test_pptx


# given ===================================================


@given("a blank slide")
def given_a_blank_slide(context):
    context.prs = Presentation(test_pptx("sld-blank"))
    context.slide = context.prs.slides[0]


@given("a notes slide")
def given_a_notes_slide(context):
    prs = Presentation(test_pptx("sld-notes"))
    context.notes_slide = prs.slides[0].notes_slide


@given("a slide")
def given_a_slide(context):
    presentation = Presentation(test_pptx("shp-shapes"))
    context.slide = presentation.slides[0]


@given("a slide having a notes slide")
def given_a_slide_having_a_notes_slide(context):
    context.slide = Presentation(test_pptx("sld-notes")).slides[0]


@given("a slide having no notes slide")
def given_a_slide_having_no_notes_slide(context):
    context.slide = Presentation(test_pptx("sld-notes")).slides[1]


@given("a slide having a title")
def given_a_slide_having_a_title(context):
    prs = Presentation(test_pptx("shp-shapes"))
    context.prs, context.slide = prs, prs.slides[0]


@given("a slide having name {name}")
def given_a_slide_having_name_name(context, name):
    slide_idx = 0 if name == "Overview" else 1
    presentation = Presentation(test_pptx("sld-slide"))
    context.slide = presentation.slides[slide_idx]


@given("a slide having slide id 256")
def given_a_slide_having_slide_id_256(context):
    presentation = Presentation(test_pptx("shp-shapes"))
    context.slide = presentation.slides[0]


@given("a Slide object based on slide_layout as slide")
def given_a_Slide_object_based_on_slide_layout_as_slide(context):
    context.slide = context.prs.slides[0]


@given("a Slide object having {def_or_ovr} background as slide")
def given_a_Slide_object_having_background_as_slide(context, def_or_ovr):
    slide_idx = {"the default": 0, "an overridden": 1}[def_or_ovr]
    context.slide = Presentation(test_pptx("sld-slide")).slides[slide_idx]


@given("a SlideLayout object as slide")
@given("a SlideLayout object as slide_layout")
def given_a_SlideLayout_object_as_slide(context):
    prs = Presentation(test_pptx("sld-slide"))
    context.slide = context.slide_layout = prs.slide_layouts[0]


@given("a SlideLayout object having name {name} as slide")
def given_a_SlideLayout_object_having_name_as_slide(context, name):
    slide_layout_idx = 0 if name == "of no explicit value" else 1
    prs = Presentation(test_pptx("sld-slide"))
    context.slide = prs.slide_layouts[slide_layout_idx]


@given("a SlideLayout object used by {which_slides} as slide_layout")
def given_a_SlideLayout_object_used_by_slides_as_slide_layout(context, which_slides):
    slide_layout_idx = {"a slide": 0, "no slides": 1}[which_slides]
    context.prs = Presentation(test_pptx("sld-slide"))
    context.slide_layout = context.prs.slide_layouts[slide_layout_idx]


@given("a SlideMaster object as slide")
@given("a SlideMaster object as slide_master")
def given_a_SlideMaster_object_as_slide(context):
    prs = Presentation(test_pptx("sld-slide"))
    context.slide = context.slide_master = prs.slide_masters[0]


# then ====================================================


@then("len(notes_slide.shapes) is {count}")
def then_len_notes_slide_shapes_is_count(context, count):
    shapes = context.notes_slide.shapes
    assert len(shapes) == int(count)


@then("notes_slide.notes_placeholder is a NotesSlidePlaceholder object")
def then_notes_slide_notes_placeholder_is_a_NotesSlidePlacehldr_obj(context):
    notes_slide = context.notes_slide
    cls_name = type(notes_slide.notes_placeholder).__name__
    assert cls_name == "NotesSlidePlaceholder", "got %s" % cls_name


@then("notes_slide.notes_text_frame is a TextFrame object")
def then_notes_slide_notes_text_frame_is_a_TextFrame_object(context):
    notes_slide = context.notes_slide
    cls_name = type(notes_slide.notes_text_frame).__name__
    assert cls_name == "TextFrame", "got %s" % cls_name


@then("notes_slide.placeholders is a NotesSlidePlaceholders object")
def then_notes_slide_placeholders_is_a_NotesSlidePlaceholders_object(context):
    notes_slide = context.notes_slide
    assert type(notes_slide.placeholders).__name__ == "NotesSlidePlaceholders"


@then("slide in slide_layout.used_by_slides is True")
def then_slide_in_slide_layout_used_by_slides_is_True(context):
    assert context.slide in context.slide_layout.used_by_slides


@then("slide.background is a _Background object")
def then_slide_background_is_a_Background_object(context):
    cls_name = context.slide.background.__class__.__name__
    assert cls_name == "_Background", "slide.background is a %s object" % cls_name


@then("slide.follow_master_background is {value}")
def then_slide_follow_master_background_is_value(context, value):
    expected_value = {"True": True, "False": False}[value]
    actual_value = context.slide.follow_master_background
    assert actual_value is expected_value, (
        "slide.follow_master_background is %s" % actual_value
    )


@then("slide.has_notes_slide is {value}")
def then_slide_has_notes_slide_is_value(context, value):
    expected_value = {"True": True, "False": False}[value]
    slide = context.slide
    assert slide.has_notes_slide is expected_value


@then("slide.name is {value}")
def then_slide_name_is_value(context, value):
    expected_name = "" if value == "the empty string" else value
    actual_name = context.slide.name
    assert actual_name == expected_name, "slide.name == %s" % actual_name


@then("slide.notes_slide is a NotesSlide object")
def then_slide_notes_slide_is_a_NotesSlide_object(context):
    notes_slide = context.notes_slide = context.slide.notes_slide
    assert type(notes_slide).__name__ == "NotesSlide"


@then("slide.placeholders is a {clsname} object")
def then_slide_placeholders_is_a_clsname_object(context, clsname):
    actual_clsname = context.slide.placeholders.__class__.__name__
    expected_clsname = clsname
    assert actual_clsname == expected_clsname, (
        "slide.placeholders is a %s object" % actual_clsname
    )


@then("slide.shapes is a {clsname} object")
def then_slide_shapes_is_a_clsname_object(context, clsname):
    actual_clsname = context.slide.shapes.__class__.__name__
    expected_clsname = clsname
    assert actual_clsname == expected_clsname, (
        "slide.shapes is a %s object" % actual_clsname
    )


@then("slide.slide_id is 256")
def then_slide_slide_id_is_256(context):
    slide = context.slide
    assert slide.slide_id == 256


@then("slide.slide_layout is the one passed in the call")
def then_slide_slide_layout_is_the_one_passed_in_the_call(context):
    slide = context.prs.slides[3]
    assert slide.slide_layout == context.slide_layout


@then("slide_layout.slide_master is a SlideMaster object")
def then_slide_layout_slide_master_is_a_SlideMaster_object(context):
    slide_layout = context.slide_layout
    assert type(slide_layout.slide_master).__name__ == "SlideMaster"


@then("slide_master.slide_layouts is a SlideLayouts object")
def then_slide_master_slide_layouts_is_a_SlideLayouts_object(context):
    slide_master = context.slide_master
    assert type(slide_master.slide_layouts).__name__ == "SlideLayouts"


@then("slide_layout.used_by_slides == ()")
def then_slide_layout_used_by_slides_eq_empty_tuple(context):
    assert context.slide_layout.used_by_slides == ()



# encoding: utf-8

"""
Utility functions wrapping the excellent *mock* library.
"""

from __future__ import absolute_import, print_function

import sys

if sys.version_info >= (3, 3):
    from unittest import mock  # noqa
    from unittest.mock import ANY, call, MagicMock  # noqa
    from unittest.mock import create_autospec, Mock, mock_open, patch, PropertyMock
else:
    import mock  # noqa
    from mock import ANY, call, MagicMock  # noqa
    from mock import create_autospec, Mock, mock_open, patch, PropertyMock


def class_mock(request, q_class_name, autospec=True, **kwargs):
    """
    Return a mock patching the class with qualified name *q_class_name*.
    The mock is autospec'ed based on the patched class unless the optional
    argument *autospec* is set to False. Any other keyword arguments are
    passed through to Mock(). Patch is reversed after calling test returns.
    """
    _patch = patch(q_class_name, autospec=autospec, **kwargs)
    request.addfinalizer(_patch.stop)
    return _patch.start()


def cls_attr_mock(request, cls, attr_name, name=None, **kwargs):
    """
    Return a mock for attribute *attr_name* on *cls* where the patch is
    reversed after pytest uses it.
    """
    name = request.fixturename if name is None else name
    _patch = patch.object(cls, attr_name, name=name, **kwargs)
    request.addfinalizer(_patch.stop)
    return _patch.start()


def function_mock(request, q_function_name, **kwargs):
    """
    Return a mock patching the function with qualified name
    *q_function_name*. Patch is reversed after calling test returns.
    """
    _patch = patch(q_function_name, **kwargs)
    request.addfinalizer(_patch.stop)
    return _patch.start()


def initializer_mock(request, cls, **kwargs):
    """
    Return a mock for the __init__ method on *cls* where the patch is
    reversed after pytest uses it.
    """
    _patch = patch.object(cls, "__init__", return_value=None, **kwargs)
    request.addfinalizer(_patch.stop)
    return _patch.start()


def instance_mock(request, cls, name=None, spec_set=True, **kwargs):
    """
    Return a mock for an instance of *cls* that draws its spec from the class
    and does not allow new attributes to be set on the instance. If *name* is
    missing or |None|, the name of the returned |Mock| instance is set to
    *request.fixturename*. Additional keyword arguments are passed through to
    the Mock() call that creates the mock.
    """
    name = name if name is not None else request.fixturename
    return create_autospec(cls, _name=name, spec_set=spec_set, instance=True, **kwargs)


def loose_mock(request, name=None, **kwargs):
    """
    Return a "loose" mock, meaning it has no spec to constrain calls on it.
    Additional keyword arguments are passed through to Mock(). If called
    without a name, it is assigned the name of the fixture.
    """
    if name is None:
        name = request.fixturename
    return Mock(name=name, **kwargs)


def method_mock(request, cls, method_name, **kwargs):
    """
    Return a mock for method *method_name* on *cls* where the patch is
    reversed after pytest uses it.
    """
    _patch = patch.object(cls, method_name, **kwargs)
    request.addfinalizer(_patch.stop)
    return _patch.start()


def open_mock(request, module_name, **kwargs):
    """
    Return a mock for the builtin `open()` method in *module_name*.
    """
    target = "%s.open" % module_name
    _patch = patch(target, mock_open(), create=True, **kwargs)
    request.addfinalizer(_patch.stop)
    return _patch.start()


def property_mock(request, cls, prop_name, **kwargs):
    """
    Return a mock for property *prop_name* on class *cls* where the patch is
    reversed after pytest uses it.
    """
    _patch = patch.object(cls, prop_name, new_callable=PropertyMock, **kwargs)
    request.addfinalizer(_patch.stop)
    return _patch.start()


def var_mock(request, q_var_name, **kwargs):
    """
    Return a mock patching the variable with qualified name *q_var_name*.
    Patch is reversed after calling test returns.
    """
    _patch = patch(q_var_name, **kwargs)
    request.addfinalizer(_patch.stop)
    return _patch.start()



# encoding: utf-8

"""
Test suite for pptx.oxml.autoshape module.
"""

from __future__ import absolute_import, print_function

import pytest

from pptx.enum.shapes import PP_PLACEHOLDER
from pptx.oxml.ns import nsdecls
from pptx.oxml.shapes.autoshape import CT_Shape
from pptx.oxml.shapes.shared import ST_Direction, ST_PlaceholderSize

from ..unitdata.shape import a_gd, a_prstGeom, an_avLst
from ...unitutil.cxml import element


class DescribeCT_PresetGeometry2D(object):
    def it_can_get_the_gd_elms_as_a_sequence(self, gd_lst_fixture):
        prstGeom, expected_vals = gd_lst_fixture
        actual_vals = [(gd.name, gd.fmla) for gd in prstGeom.gd_lst]
        assert actual_vals == expected_vals

    def it_can_rewrite_the_gd_elms(self, rewrite_guides_fixture_):
        prstGeom, guides, expected_xml = rewrite_guides_fixture_
        print(prstGeom.xml)
        prstGeom.rewrite_guides(guides)
        assert prstGeom.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            [],
            [("adj1", "val 111")],
            [
                ("adj2", "val 222"),
                ("adj3", "val 333"),
                ("adj4", "val 444"),
                ("adj5", "val 555"),
            ],
        ]
    )
    def gd_lst_fixture(self, request):
        expected_vals = request.param
        prstGeom = self.prstGeom_bldr("foobar", expected_vals).element
        return prstGeom, expected_vals

    @pytest.fixture(
        params=[
            ("circularArrow", 5),  # all five guides for five adj shape
            ("chevron", 0),  # empty guides for single adj shape
            ("chevron", 1),  # one guide for single adj shape
        ]
    )
    def rewrite_guides_fixture_(self, request):
        prst, gd_count = request.param

        names = ("adj1", "adj2", "adj3", "adj4", "adj5")
        vals = (111, 222, 333, 444, 555)
        fmlas = [("val %d" % v) for v in vals]
        before_vals = [("adj6", "val 666")] * 3
        after_vals = zip(names[:gd_count], fmlas[:gd_count])

        prstGeom = self.prstGeom_bldr(prst, before_vals).element
        guides = zip(names[:gd_count], vals[:gd_count])
        expected_xml = self.prstGeom_bldr(prst, after_vals).xml()

        return prstGeom, guides, expected_xml

    # fixture components ---------------------------------------------

    def prstGeom_bldr(self, prst, gd_vals):
        avLst_bldr = an_avLst()
        for name, fmla in gd_vals:
            gd_bldr = a_gd().with_name(name).with_fmla(fmla)
            avLst_bldr.with_child(gd_bldr)
        prstGeom_bldr = (
            a_prstGeom().with_nsdecls().with_prst(prst).with_child(avLst_bldr)
        )
        return prstGeom_bldr


class DescribeCT_Shape(object):
    def it_knows_how_to_create_a_new_autoshape_sp(self):
        # setup ------------------------
        id_ = 9
        name = "Rounded Rectangle 8"
        prst = "roundRect"
        left, top, width, height = 111, 222, 333, 444
        xml = (
            '<p:sp %s>\n  <p:nvSpPr>\n    <p:cNvPr id="%d" name="%s"/>\n    <'
            "p:cNvSpPr/>\n    <p:nvPr/>\n  </p:nvSpPr>\n  <p:spPr>\n    <a:xf"
            'rm>\n      <a:off x="%d" y="%d"/>\n      <a:ext cx="%d" cy="%d"/'
            '>\n    </a:xfrm>\n    <a:prstGeom prst="%s">\n      <a:avLst/>\n'
            '    </a:prstGeom>\n  </p:spPr>\n  <p:style>\n    <a:lnRef idx="1'
            '">\n      <a:schemeClr val="accent1"/>\n    </a:lnRef>\n    <a:f'
            'illRef idx="3">\n      <a:schemeClr val="accent1"/>\n    </a:fil'
            'lRef>\n    <a:effectRef idx="2">\n      <a:schemeClr val="accent'
            '1"/>\n    </a:effectRef>\n    <a:fontRef idx="minor">\n      <a:'
            'schemeClr val="lt1"/>\n    </a:fontRef>\n  </p:style>\n  <p:txBo'
            'dy>\n    <a:bodyPr rtlCol="0" anchor="ctr"/>\n    <a:lstStyle/>'
            '\n    <a:p>\n      <a:pPr algn="ctr"/>\n    </a:p>\n  </p:txBody'
            ">\n</p:sp>\n"
            % (nsdecls("a", "p"), id_, name, left, top, width, height, prst)
        )
        # exercise ---------------------
        sp = CT_Shape.new_autoshape_sp(id_, name, prst, left, top, width, height)
        # verify -----------------------
        assert sp.xml == xml

    def it_knows_how_to_create_a_new_placeholder_sp(self, new_ph_sp_fixture):
        id_, name, ph_type, orient, sz, idx, expected_xml = new_ph_sp_fixture
        sp = CT_Shape.new_placeholder_sp(id_, name, ph_type, orient, sz, idx)
        assert sp.xml == expected_xml

    def it_knows_how_to_create_a_new_textbox_sp(self):
        # setup ------------------------
        id_ = 9
        name = "TextBox 8"
        left, top, width, height = 111, 222, 333, 444
        xml = (
            '<p:sp %s>\n  <p:nvSpPr>\n    <p:cNvPr id="%d" name="%s"/>\n    '
            '<p:cNvSpPr txBox="1"/>\n    <p:nvPr/>\n  </p:nvSpPr>\n  <p:spPr'
            '>\n    <a:xfrm>\n      <a:off x="%d" y="%d"/>\n      <a:ext cx='
            '"%d" cy="%d"/>\n    </a:xfrm>\n    <a:prstGeom prst="rect">\n  '
            "    <a:avLst/>\n    </a:prstGeom>\n    <a:noFill/>\n  </p:spPr>"
            '\n  <p:txBody>\n    <a:bodyPr wrap="none">\n      <a:spAutoFit/'
            ">\n    </a:bodyPr>\n    <a:lstStyle/>\n    <a:p/>\n  </p:txBody"
            ">\n</p:sp>\n" % (nsdecls("a", "p"), id_, name, left, top, width, height)
        )
        # exercise ---------------------
        sp = CT_Shape.new_textbox_sp(id_, name, left, top, width, height)
        # verify -----------------------
        assert sp.xml == xml

    def it_knows_whether_it_is_an_autoshape(self, is_autoshape_fixture):
        sp, expected_value = is_autoshape_fixture
        assert sp.is_autoshape is expected_value

    def it_knows_whether_it_is_an_textbox(self, is_textbox_fixture):
        sp, expected_value = is_textbox_fixture
        assert sp.is_textbox is expected_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            # autoshape
            ("p:sp/(p:nvSpPr/p:cNvSpPr, p:spPr/a:prstGeom)", True),
            # placeholder
            ("p:sp/(p:nvSpPr/p:nvPr/p:ph, p:spPr)", False),
            # textbox
            ("p:sp/(p:nvSpPr/p:cNvSpPr{txBox=1}, p:spPr/a:prstGeom)", False),
        ]
    )
    def is_autoshape_fixture(self, request):
        sp_cxml, expected_value = request.param
        sp = element(sp_cxml)
        return sp, expected_value

    @pytest.fixture(
        params=[
            # autoshape
            ("p:sp/(p:nvSpPr/p:cNvSpPr, p:spPr/a:prstGeom)", False),
            # placeholder
            ("p:sp/(p:nvSpPr/(p:nvPr/p:ph, p:cNvSpPr), p:spPr)", False),
            # textbox
            ("p:sp/(p:nvSpPr/p:cNvSpPr{txBox=1}, p:spPr/a:prstGeom)", True),
        ]
    )
    def is_textbox_fixture(self, request):
        sp_cxml, expected_value = request.param
        sp = element(sp_cxml)
        return sp, expected_value

    @pytest.fixture(
        params=[
            (
                2,
                "Title 1",
                PP_PLACEHOLDER.CENTER_TITLE,
                ST_Direction.HORZ,
                ST_PlaceholderSize.FULL,
                0,
                ' type="ctrTitle"',
            ),
            (
                3,
                "Date Placeholder 2",
                PP_PLACEHOLDER.DATE,
                ST_Direction.HORZ,
                ST_PlaceholderSize.HALF,
                10,
                ' type="dt" sz="half" idx="10"',
            ),
            (
                4,
                "Vertical Subtitle 3",
                PP_PLACEHOLDER.SUBTITLE,
                ST_Direction.VERT,
                ST_PlaceholderSize.FULL,
                1,
                ' type="subTitle" orient="vert" idx="1"',
            ),
            (
                5,
                "Table Placeholder 4",
                PP_PLACEHOLDER.TABLE,
                ST_Direction.HORZ,
                ST_PlaceholderSize.QUARTER,
                14,
                ' type="tbl" sz="quarter" idx="14"',
            ),
            (
                6,
                "Slide Number Placeholder 5",
                PP_PLACEHOLDER.SLIDE_NUMBER,
                ST_Direction.HORZ,
                ST_PlaceholderSize.QUARTER,
                12,
                ' type="sldNum" sz="quarter" idx="12"',
            ),
            (
                7,
                "Footer Placeholder 6",
                PP_PLACEHOLDER.FOOTER,
                ST_Direction.HORZ,
                ST_PlaceholderSize.QUARTER,
                11,
                ' type="ftr" sz="quarter" idx="11"',
            ),
            (
                8,
                "Content Placeholder 7",
                PP_PLACEHOLDER.OBJECT,
                ST_Direction.HORZ,
                ST_PlaceholderSize.FULL,
                15,
                ' idx="15"',
            ),
        ]
    )
    def new_ph_sp_fixture(self, request):
        id_, name, ph_type, orient, sz, idx, expected_attrs = request.param
        expected_xml_tmpl = (
            "<p:sp %s>\n"
            "  <p:nvSpPr>\n"
            '    <p:cNvPr id="%s" name="%s"/>\n'
            "    <p:cNvSpPr>\n"
            '      <a:spLocks noGrp="1"/>\n'
            "    </p:cNvSpPr>\n"
            "    <p:nvPr>\n"
            "      <p:ph%s/>\n"
            "    </p:nvPr>\n"
            "  </p:nvSpPr>\n"
            "  <p:spPr/>\n"
            "%s"
            "</p:sp>\n" % (nsdecls("a", "p"), "%d", "%s", "%s", "%s")
        )
        txBody_snippet = (
            "  <p:txBody>\n"
            "    <a:bodyPr/>\n"
            "    <a:lstStyle/>\n"
            "    <a:p/>\n"
            "  </p:txBody>\n"
        )
        txBody_str = txBody_snippet if id_ in (2, 4, 8) else ""
        expected_values = (id_, name, expected_attrs, txBody_str)
        expected_xml = expected_xml_tmpl % expected_values
        return id_, name, ph_type, orient, sz, idx, expected_xml



# encoding: utf-8

"""Test suite for pptx.dml.effect module."""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from pptx.dml.effect import ShadowFormat

from ..unitutil.cxml import element, xml


class DescribeShadowFormat(object):
    def it_knows_whether_it_inherits(self, inherit_get_fixture):
        shadow, expected_value = inherit_get_fixture
        inherit = shadow.inherit
        assert inherit is expected_value

    def it_can_change_whether_it_inherits(self, inherit_set_fixture):
        shadow, value, expected_xml = inherit_set_fixture
        shadow.inherit = value
        assert shadow._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            ("p:spPr", True),
            ("p:spPr/a:effectLst", False),
            ("p:grpSpPr", True),
            ("p:grpSpPr/a:effectLst", False),
        ]
    )
    def inherit_get_fixture(self, request):
        cxml, expected_value = request.param
        shadow = ShadowFormat(element(cxml))
        return shadow, expected_value

    @pytest.fixture(
        params=[
            ("p:spPr{a:b=c}", False, "p:spPr{a:b=c}/a:effectLst"),
            ("p:grpSpPr{a:b=c}", False, "p:grpSpPr{a:b=c}/a:effectLst"),
            ("p:spPr{a:b=c}/a:effectLst", True, "p:spPr{a:b=c}"),
            ("p:grpSpPr{a:b=c}/a:effectLst", True, "p:grpSpPr{a:b=c}"),
            ("p:spPr", True, "p:spPr"),
            ("p:grpSpPr", True, "p:grpSpPr"),
            ("p:spPr/a:effectLst", False, "p:spPr/a:effectLst"),
            ("p:grpSpPr/a:effectLst", False, "p:grpSpPr/a:effectLst"),
        ]
    )
    def inherit_set_fixture(self, request):
        cxml, value, expected_cxml = request.param
        shadow = ShadowFormat(element(cxml))
        expected_value = xml(expected_cxml)
        return shadow, value, expected_value



# encoding: utf-8

"""
Test suite for pptx.util module.
"""

from __future__ import absolute_import

import pytest

from pptx.compat import to_unicode
from pptx.util import Length, Centipoints, Cm, Emu, Inches, Mm, Pt


def test_to_unicode_raises_on_non_string():
    """to_unicode(text) raises on *text* not a string"""
    with pytest.raises(TypeError):
        to_unicode(999)


class DescribeLength(object):
    def it_can_construct_from_convenient_units(self, construct_fixture):
        UnitCls, units_val, emu = construct_fixture
        length = UnitCls(units_val)
        assert isinstance(length, Length)
        assert length == emu

    def it_can_self_convert_to_convenient_units(self, units_fixture):
        emu, units_prop_name, expected_length_in_units = units_fixture
        length = Length(emu)
        length_in_units = getattr(length, units_prop_name)
        assert length_in_units == expected_length_in_units

    # fixtures -------------------------------------------------------

    @pytest.fixture(
        params=[
            (Length, 914400, 914400),
            (Inches, 1.1, 1005840),
            (Centipoints, 12.5, 1587),
            (Cm, 2.53, 910799),
            (Emu, 9144.9, 9144),
            (Mm, 13.8, 496800),
            (Pt, 24.5, 311150),
        ]
    )
    def construct_fixture(self, request):
        UnitCls, units_val, emu = request.param
        return UnitCls, units_val, emu

    @pytest.fixture(
        params=[
            (914400, "inches", 1.0),
            (914400, "centipoints", 7200.0),
            (914400, "cm", 2.54),
            (914400, "emu", 914400),
            (914400, "mm", 25.4),
            (914400, "pt", 72.0),
        ]
    )
    def units_fixture(self, request):
        emu, units_prop_name, expected_length_in_units = request.param
        return emu, units_prop_name, expected_length_in_units
